/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef UTIL_H
#define UTIL_H

#include <stdint.h>
#include <string>
#include <cstdio>
#include <cstring>
#include <cstdlib>

using std::string;

namespace Spellathon
{
	
	void *	memalloc(size_t newsize);
	void *	memresize(void *mem, size_t newsize);
	
	void	spth_error(bool syserr, const char *msg, ...) throw (string);
	
	/* returns a random no. in the range [low,high) */
	static inline int
	randnum(const int low, const int high)
	{
		double	l = (double)low;
		double	h = (double)high;
	
# ifdef WIN32
		return (int)(l + (h - l) * (rand() / (RAND_MAX + 1.0)));
# else
		return (int)(l + (h - l) * (random() / (((1U<<31)-1) + 1.0)));
# endif /* WIN32 */
	}
	
	static inline void 
	swapch(char *a, char *b) 
	{ 
		char t; 
		t = *b; 
		*b = *a; 
		*a = t; 
	} 
	
	static inline void 
	swapi(int *a, int *b) 
	{ 
		int t; 
		t = *b; 
		*b = *a; 
		*a = t; 
	} 
	
	static inline void 
	spinner(void) 
	{
		const char spin[] = {'|','/','-','\\'};
		static char c = 0;
	
		printf("\b%c", spin[++c & 0x3]);
		fflush(stdout);
	}
	
	/* performance critical routine */
	static inline void 
	sign(char* w, int sz)
	{
		int		i, j;
		char	val;
	
		/* insertion sort works well here since sz is 
		 * very small (<= 7) */
		for (i = 1; i < sz; ++i) {
	        val = w[i];
			j = i;
			while (j > 0 && val < w[j-1]) {
				w[j] = w[j-1];
				--j;
	       	}
			w[j] = val;
		}
	}
	
	/* performance critical routine */
	static inline int
	strcmp7(const char *p1, const char *p2)
	{
				int i = 0;

				if (p1[0] != p2[0]) { goto out; }
				if (p1[1] != p2[1]) { i = 1; goto out; }
				if (p1[2] != p2[2]) { i = 2; goto out; }
				if (p1[3] != p2[3]) { i = 3; goto out; }
				if (p1[4] != p2[4]) { i = 4; goto out; }
				if (p1[5] != p2[5]) { i = 5; goto out; }
				if (p1[6] != p2[6]) { i = 6; }
out:
				return p1[i] - p2[i];
	}

/* performance critical routine */
#define strcmp7_eq(p1, p2) ((*((uint64_t*)p1)) == (*((uint64_t*)p2)))
#define strcpy7(p1, p2) ((*((uint64_t*)p1)) = (*((uint64_t*)p2)))

}

#endif /* UTIL_H */


